package dev.shadowsoffire.apotheosis.spawn.compat;

import java.util.stream.Stream;
import java.util.function.Function;
import java.util.Collection;
import java.util.Iterator;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.gui.Font;
import net.minecraft.client.gui.screens.Screen;
import java.util.ArrayList;
import net.minecraft.network.chat.FormattedText;
import dev.shadowsoffire.apotheosis.spawn.modifiers.StatModifier;
import java.util.Objects;
import net.minecraft.ChatFormatting;
import java.util.List;
import java.util.Arrays;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiGraphics;
import mezz.jei.api.gui.ingredient.IRecipeSlotsView;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.item.crafting.Ingredient;
import mezz.jei.api.recipe.RecipeIngredientRole;
import mezz.jei.api.recipe.IFocusGroup;
import mezz.jei.api.gui.builder.IRecipeLayoutBuilder;
import mezz.jei.api.ingredients.IIngredientType;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import mezz.jei.api.constants.VanillaTypes;
import mezz.jei.api.helpers.IGuiHelper;
import net.minecraft.network.chat.Component;
import mezz.jei.api.gui.drawable.IDrawable;
import mezz.jei.api.recipe.RecipeType;
import net.minecraft.resources.ResourceLocation;
import dev.shadowsoffire.apotheosis.spawn.modifiers.SpawnerModifier;
import mezz.jei.api.recipe.category.IRecipeCategory;

public class SpawnerCategory implements IRecipeCategory<SpawnerModifier>
{
    public static final ResourceLocation TEXTURES;
    public static final ResourceLocation UID;
    public static final RecipeType<SpawnerModifier> TYPE;
    private IDrawable bg;
    private IDrawable icon;
    private Component title;
    
    public SpawnerCategory(final IGuiHelper helper) {
        this.bg = (IDrawable)helper.drawableBuilder(SpawnerCategory.TEXTURES, 0, 0, 169, 75).build();
        this.icon = helper.createDrawableIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)new ItemStack((ItemLike)Items.f_42007_));
        this.title = (Component)Component.m_237115_("title.apotheosis.spawner");
    }
    
    public RecipeType<SpawnerModifier> getRecipeType() {
        return SpawnerCategory.TYPE;
    }
    
    public Component getTitle() {
        return this.title;
    }
    
    public IDrawable getBackground() {
        return this.bg;
    }
    
    public void setRecipe(final IRecipeLayoutBuilder builder, final SpawnerModifier recipe, final IFocusGroup focuses) {
        builder.addSlot(RecipeIngredientRole.INPUT, 11, 11).addIngredients(recipe.getMainhandInput());
        if (recipe.getOffhandInput() != Ingredient.f_43901_) {
            builder.addSlot(RecipeIngredientRole.INPUT, 11, 48).addIngredients(recipe.getOffhandInput());
        }
        builder.addInvisibleIngredients(RecipeIngredientRole.CATALYST).addIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)new ItemStack((ItemLike)Blocks.f_50085_));
        builder.addInvisibleIngredients(RecipeIngredientRole.OUTPUT).addIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)new ItemStack((ItemLike)Blocks.f_50085_));
    }
    
    public IDrawable getIcon() {
        return this.icon;
    }
    
    public void draw(final SpawnerModifier recipe, final IRecipeSlotsView recipeSlotsView, final GuiGraphics gfx, final double mouseX, final double mouseY) {
        if (recipe.getOffhandInput() == Ingredient.f_43901_) {
            gfx.m_280398_(SpawnerCategory.TEXTURES, 1, 31, 0, 0.0f, 88.0f, 28, 34, 256, 256);
        }
        final Screen scn = Minecraft.m_91087_().f_91080_;
        final Font font = Minecraft.m_91087_().f_91062_;
        if (scn == null) {
            return;
        }
        if (mouseX >= -1.0 && mouseX < 9.0 && mouseY >= 13.0 && mouseY < 25.0) {
            gfx.m_280398_(SpawnerCategory.TEXTURES, -1, 13, 0, 0.0f, 75.0f, 10, 12, 256, 256);
            gfx.m_280666_(font, (List)Arrays.asList((Component)Component.m_237115_("misc.apotheosis.mainhand")), (int)mouseX, (int)mouseY);
        }
        else if (mouseX >= -1.0 && mouseX < 9.0 && mouseY >= 50.0 && mouseY < 62.0 && recipe.getOffhandInput() != Ingredient.f_43901_) {
            gfx.m_280398_(SpawnerCategory.TEXTURES, -1, 50, 0, 0.0f, 75.0f, 10, 12, 256, 256);
            gfx.m_280666_(font, (List)Arrays.asList((Component)Component.m_237115_("misc.apotheosis.offhand"), (Component)Component.m_237115_("misc.apotheosis.not_consumed").m_130940_(ChatFormatting.GRAY)), (int)mouseX, (int)mouseY);
        }
        else if (mouseX >= 33.0 && mouseX < 49.0 && mouseY >= 30.0 && mouseY < 46.0) {
            gfx.m_280666_(font, (List)Arrays.asList((Component)Component.m_237115_("misc.apotheosis.rclick_spawner")), (int)mouseX, (int)mouseY);
        }
        final PoseStack mvStack = gfx.m_280168_();
        mvStack.m_85836_();
        mvStack.m_85837_(0.0, 0.5, 0.0);
        gfx.m_280203_(new ItemStack((ItemLike)Items.f_42007_), 31, 29);
        mvStack.m_85849_();
        final int n = 37;
        final int size = recipe.getStatModifiers().size();
        Objects.requireNonNull(font);
        int top = n - size * (9 + 2) / 2 + 2;
        final int left = 168;
        for (StatModifier<?> s : recipe.getStatModifiers()) {
            String value = s.value().toString();
            if ("true".equals(value)) {
                value = "+";
            }
            else if ("false".equals(value)) {
                value = "-";
            }
            else {
                final Object value2 = s.value();
                if (value2 instanceof final Number num) {
                    if (num.intValue() > 0) {
                        value = "+" + value;
                    }
                }
            }
            final Component msg = (Component)Component.m_237110_("misc.apotheosis.concat", new Object[] { value, s.stat().name() });
            final int width = font.m_92852_((FormattedText)msg);
            boolean b = false;
            Label_0615: {
                if (mouseX >= left - width && mouseX < left && mouseY >= top) {
                    final int n2 = top;
                    Objects.requireNonNull(font);
                    if (mouseY < n2 + 9 + 1) {
                        b = true;
                        break Label_0615;
                    }
                }
                b = false;
            }
            final boolean hover = b;
            gfx.m_280614_(font, msg, left - font.m_92852_((FormattedText)msg), top, hover ? 8421631 : 3355443, false);
            int maxWidth = Minecraft.m_91087_().m_91268_().m_85445_();
            maxWidth = maxWidth - (maxWidth - 210) / 2 - 210;
            if (hover) {
                final List<Component> list = new ArrayList<Component>();
                list.add((Component)s.stat().name().m_130944_(new ChatFormatting[] { ChatFormatting.GREEN, ChatFormatting.UNDERLINE }));
                list.add((Component)s.stat().desc().m_130940_(ChatFormatting.GRAY));
                if (s.value() instanceof Number) {
                    if (((Number)s.min()).intValue() > 0 || ((Number)s.max()).intValue() != Integer.MAX_VALUE) {
                        list.add((Component)Component.m_237113_(" "));
                    }
                    if (((Number)s.min()).intValue() > 0) {
                        list.add((Component)Component.m_237110_("misc.apotheosis.min_value", new Object[] { s.min() }).m_130940_(ChatFormatting.GRAY));
                    }
                    if (((Number)s.max()).intValue() != Integer.MAX_VALUE) {
                        list.add((Component)Component.m_237110_("misc.apotheosis.max_value", new Object[] { s.max() }).m_130940_(ChatFormatting.GRAY));
                    }
                }
                renderComponentTooltip(scn, gfx, list, left + 6, (int)mouseY, maxWidth, font);
            }
            final int n3 = top;
            Objects.requireNonNull(font);
            top = n3 + (9 + 2);
        }
    }
    
    private static void renderComponentTooltip(final Screen scn, final GuiGraphics gfx, final List<Component> list, final int x, final int y, final int maxWidth, final Font font) {
        final List<FormattedText> text = list.stream().map(c -> font.m_92865_().m_92414_((FormattedText)c, maxWidth, c.m_7383_())).flatMap((Function<? super Object, ? extends Stream<? extends FormattedText>>)Collection::stream).toList();
        gfx.renderComponentTooltip(font, (List)text, x, y, ItemStack.f_41583_);
    }
    
    static {
        TEXTURES = new ResourceLocation("apotheosis", "textures/gui/spawner_jei.png");
        UID = new ResourceLocation("apotheosis", "spawner_modifiers");
        TYPE = RecipeType.create("apotheosis", "spawner_modifiers", (Class)SpawnerModifier.class);
    }
}
